import orjson
import mwparserfromhell as mwp
import tqdm
from nltk.tokenize import wordpunct_tokenize
import gzip
from gobbet import Wordlist
import struct


def parse_dump(filename):
    def getuncompressedsize(filename):
        with open(filename, "rb") as f:
            f.seek(-4, 2)
            return struct.unpack("I", f.read(4))[0]

    f = gzip.open(filename)
    articles = []
    headlines = []
    paragraphs = []
    threshold = 2

    words = Wordlist()
    # This ain't real json.
    with tqdm.tqdm(
        total=getuncompressedsize(filename), unit_scale=True, desc="parsing"
    ) as bar:
        for line in f:
            bar.update(len(line))
            article = orjson.loads(line)
            if "index" in article:
                continue
            # Just store text here
            text = article["text"]
            this_words = filter(lambda x: len(x) > 2, wordpunct_tokenize(text))
            words.update(this_words)

            if article["namespace"] != 0:
                continue
            headlines.append(article["title"])
            plain_text = mwp.parse(article["source_text"]).strip_code()
            for para in plain_text.split("\n\n"):
                if not para.startswith("Category:"):
                    paragraphs.append(para)

    return {
        "paragraphs": paragraphs,
        "headlines": headlines,
        "words": words,
    }



from gobbet.wordlist import Wordlist
from gobbet.downloader import news_dir, get_news_json
from gobbet.parser import parse_dump
import pickle
import os
from youseedee import parse_file_ranges
from collections import defaultdict


unicode_ranges = defaultdict(list)
for s, e, c in parse_file_ranges("Scripts.txt"):
    unicode_ranges[c].append(range(s, e + 1))


def filter_length(items, threshold=3):
    return [x for x in items if len(x) >= threshold]


def filter_unicodes(items, codepoint_ranges):
    def _included_letter(l):
        return any(ord(l) in r for r in codepoint_ranges)

    def _included(word):
        return all(_included_letter(l) for l in word)

    return [x for x in items if _included(x)]


def get_news(lang="en", force=False):
    news_path = os.path.join(news_dir(), f"{lang}.pickle")
    if os.path.isfile(news_path) and not force:
        with open(news_path, "rb") as pickle_file:
            news = pickle.load(pickle_file)
        return news["words"], news["headlines"], news["paragraphs"]

    news = parse_dump(get_news_json(lang, force))

    with open(news_path, "wb") as pickle_file:
        pickle.dump(news, pickle_file)
    return news["words"], news["headlines"], news["paragraphs"]



import htmllistparse
import tqdm
import os
import requests

url = "https://dumps.wikimedia.org/other/cirrussearch/current/"


def news_dir():
    newsdir = os.path.expanduser("~/.gobbet")
    if not os.path.isdir(newsdir):
        os.mkdir(newsdir)
    return newsdir


def get_available():
    cwd, listing = htmllistparse.fetch_listing(url, timeout=30)
    return {
        l.name[0:2]: l.name
        for l in listing
        if "wikinews" in l.name and "content" in l.name
    }


def download_news(lang="en"):
    news_path = os.path.join(news_dir(), f"{lang}.json.gz")
    available = get_available()
    if lang not in available:
        raise ValueError("No news for language '%s'" % lang)

    with open(news_path, "wb") as f:
        response = requests.get(url + available[lang], stream=True)
        total_length = response.headers.get("content-length")

        if total_length is None:  # no content length header
            f.write(response.content)
        else:
            dl = 0
            with tqdm.tqdm(total=int(total_length), desc="downloading") as bar:
                for data in response.iter_content(chunk_size=4096):
                    bar.update(len(data))
                    f.write(data)


def get_news_json(lang="en", force=False):
    news_path = os.path.join(news_dir(), f"{lang}.json.gz")
    if not os.path.isfile(news_path) or force:
        download_news(lang)
    return news_path



from collections import Counter, defaultdict

def pairwise(a):
   return zip(a[::2], a[1::2])


class Wordlist(Counter):
	def bigrams(self):
		bigrams = defaultdict(set)
		for word in self.keys():
			for a,b in pairwise(word):
				bigrams[a+b].add(word)
		return bigrams

	def filter_popularity(self, threshold=3):
		return Wordlist({x: count for x, count in self.items() if count >= threshold})

	def filter_length(self, threshold=3):
		return Wordlist({x: count for x, count in self.items() if len(x) >= threshold})

	def filter_unicodes(self, codepoint_ranges):
		def _included_letter(l):
			return any(ord(l) in r for r in codepoint_ranges)
		def _included(word):
			return all(_included_letter(l) for l in word)

		return Wordlist({x: count for x, count in self.items() if _included(x)})
